package uk.robware.vanlights;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattService;
import android.app.Service;
import android.content.Intent;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;

import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class LightsService extends Service {
	private final IBinder binder = new LightsBinder();
	private Map<String, Light> _lights= new HashMap<>();
	private Map<Integer, Light> _lightsById= new HashMap<>();
	private MainActivity _mainActivity;

	public LightsService(){
		_lights.put("seating", new Light((byte) 1, "seating"));
		_lights.put("kitchen", new Light((byte) 2, "kitchen"));
		_lights.put("bathroom", new Light((byte) 3, "bathroom"));
		_lights.put("awning", new Light((byte) 4, "awning"));

		for (Light light : _lights.values()) {
			_lightsById.put(Integer.valueOf(light.id), light);
		}
	}

	public class LightsBinder extends Binder {
		LightsService getService() {
			return LightsService.this;
		}
	}

	@Override
	public IBinder onBind(Intent intent) {
		return binder;
	}
	private Handler handler = new Handler();
    BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
    BluetoothDevice _device = bluetoothAdapter.getRemoteDevice("9C:9C:1F:E3:C0:D6");
    private BluetoothGattCharacteristic _characteristic;
    private BluetoothGatt _gatt;
    BluetoothGattCallback gattCallback=new BluetoothGattCallback() {

        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            if (status!=133) {
                gatt.discoverServices();
                //gatt.requestMtu(517);
                _gatt = gatt;
            }else{
                handler.postDelayed(()->connect(), 500);
            }
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            BluetoothGattService service = gatt.getService(UUID.fromString("4fafc201-1fb5-459e-8fcc-c5c9c331914b"));
            _characteristic = service.getCharacteristic(UUID.fromString("beb5483e-36e1-4688-b7f5-ea07361b26a8"));
            gatt.setCharacteristicNotification(_characteristic, true);
            gatt.readCharacteristic(_characteristic);
        }

		@Override
		public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
			super.onCharacteristicRead(gatt, characteristic, status);
			readControllerData(characteristic);
		}

		@Override
		public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
			super.onCharacteristicChanged(gatt, characteristic);
			readControllerData(characteristic);
		}
	};

	public int bytesToInt(byte[] bytes) {
		ByteBuffer buffer = ByteBuffer.allocate(Integer.BYTES);
		buffer.put(bytes);
		buffer.flip();//need flip
		return buffer.getInt();
	}

	public float bytesToFloat(byte[] bytes) {
		ByteBuffer buffer = ByteBuffer.allocate(Float.BYTES);
		buffer.put(bytes);
		buffer.flip();//need flip
		return buffer.getFloat();
	}

	private void readControllerData(BluetoothGattCharacteristic characteristic) {
//		byte[] value = characteristic.getValue();
//		if (value.length == 0)
//			return;
//
//		for (int i=0; i<value.length;i+=10){
//			Light light=_lightsById.get(Integer.valueOf(value[i]));
//			light.on=value[i+1]==1;
//			light.brightness=bytesToFloat(new byte[]{value[i+2],value[i+3],value[i+4],value[i+5]});
//			light.timer=bytesToInt(new byte[]{value[i+6],value[i+7],value[i+8],value[i+9]});
//		}
//
//		if (_mainActivity!=null)
//			_mainActivity.dataReadCallback(_lights);
	}

	public void registerUI(MainActivity mainActivity){
		_mainActivity=mainActivity;
		_mainActivity.dataReadCallback(_lights);
	}

	public void connect(){
        _device.connectGatt(this, false, gattCallback);
    }

	public void disconnect() {
    	_gatt.disconnect();
	}

	public byte[] intToBytes(int x) {
		ByteBuffer buffer = ByteBuffer.allocate(Integer.BYTES);
		buffer.putInt(x);
		return buffer.array();
	}

	public byte[] floatToBytes(float x) {
		ByteBuffer buffer = ByteBuffer.allocate(Float.BYTES);
		buffer.putFloat(x);
		return buffer.array();
	}

	private byte[] getLightDataAsBytes() {
		byte[] data=new byte[10 * _lights.size()];

		int dataIndex=0;
		for (Light light : _lights.values()) {
			data[dataIndex]= light.id;
			data[dataIndex+1]= (byte) (light.on?1:0);

			int brightnessByteIndex=0;
			byte[] brightnessToBytes = intToBytes((int) (light.brightness*100));
			for (byte brightnessByte: brightnessToBytes) {
				data[dataIndex + 2 + brightnessByteIndex] = brightnessByte;
				brightnessByteIndex++;
			}

			int timerByteIndex=0;
			byte[] timerToBytes = intToBytes(light.timer);
			for (byte timerByte: timerToBytes) {
				data[dataIndex + 6 + timerByteIndex] = timerByte;
				timerByteIndex++;
			}

			dataIndex+=10;
		}

		return data;
	}

	public boolean sendUpdate() {
		byte[] data=getLightDataAsBytes();
		return write(data);
	}

	public void updateState(String name, boolean state) {
		Light light = _lights.get(name);
		light.on=state;
		light.timer=0;
	}

	public void updateBrightness(String name, float value) {
		Light light = _lights.get(name);
		light.brightness=value;
	}

	public void updateTimer(String name, int totalSeconds) {
		Light light=_lights.get(name);
		light.timer=totalSeconds;
	}

    private boolean write(byte[] value){
    	if (_characteristic==null || _gatt==null)
    		return false;
        _characteristic.setValue(value);
        _gatt.writeCharacteristic(_characteristic);
        return true;
    }
}