package uk.robware.vanlights;

import android.Manifest;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.View;
import android.widget.NumberPicker;
import android.widget.SeekBar;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import java.util.Map;

public class MainActivity extends AppCompatActivity {
	private static final int ACCESS_COARSE_LOCATION_REQUEST = 2;

	private boolean hasPermissions() {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
			if (getApplicationContext().checkSelfPermission(Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
				requestPermissions(new String[]{
						Manifest.permission.ACCESS_COARSE_LOCATION,
						Manifest.permission.ACCESS_FINE_LOCATION,
						Manifest.permission.ACCESS_BACKGROUND_LOCATION
				}, ACCESS_COARSE_LOCATION_REQUEST);
				return false;
			}
		}
		return true;
	}

	private LightsService _lightsService;
	private boolean _lightsBound;
	private ServiceConnection _serviceConnection = new ServiceConnection() {
		@Override
		public void onServiceConnected(ComponentName className, IBinder service) {
			// We've bound to LocalService, cast the IBinder and get LocalService instance
			LightsService.LightsBinder binder = (LightsService.LightsBinder) service;
			_lightsService = binder.getService();
			registerUiToLightsService();
			_lightsBound = true;
			connect();
			Log.d("MAIN", "Lights service connected");
		}

		@Override
		public void onServiceDisconnected(ComponentName arg0) {
			_lightsBound = false;
			Log.d("MAIN", "Lights service disconnected");
		}
	};

	private void registerUiToLightsService() {
		_lightsService.registerUI(this);
	}

	public void dataReadCallback(Map<String, Light> lights){
		int kitchen = (int) (lights.get("kitchen").brightness * 100);
		((SeekBar)findViewById(R.id.kitchen_brightness)).setProgress(kitchen);
		((SeekBar)findViewById(R.id.seating_brightness)).setProgress((int) (lights.get("seating").brightness*100));
		((SeekBar)findViewById(R.id.bathroom_brightness)).setProgress((int) (lights.get("bathroom").brightness*100));
		((SeekBar)findViewById(R.id.awning_brightness)).setProgress((int) (lights.get("awning").brightness*100));

		((NumberPicker)findViewById(R.id.seating_timer_minutes)).setValue(lights.get("seating").timer/60);
		((NumberPicker)findViewById(R.id.seating_timer_seconds)).setValue(lights.get("seating").timer%60);
		((NumberPicker)findViewById(R.id.kitchen_timer_minutes)).setValue(lights.get("kitchen").timer/60);
		((NumberPicker)findViewById(R.id.awning_timer_minutes)).setValue(lights.get("awning").timer/60);
		((NumberPicker)findViewById(R.id.kitchen_timer_seconds)).setValue(lights.get("kitchen").timer%60);
		((NumberPicker)findViewById(R.id.bathroom_timer_minutes)).setValue(lights.get("bathroom").timer/60);
		((NumberPicker)findViewById(R.id.bathroom_timer_seconds)).setValue(lights.get("bathroom").timer%60);
		((NumberPicker)findViewById(R.id.awning_timer_seconds)).setValue(lights.get("awning").timer%60);

		findViewById(R.id.scrollView).setVisibility(View.VISIBLE);
	}

	private void connect() {
		if (!_lightsBound) return;
		_lightsService.connect();
	}

	private void disconnect() {
		if (!_lightsBound) return;
		_lightsService.disconnect();
	}

	private void send(){
		if (!_lightsService.sendUpdate()) {
			Toast.makeText(getApplicationContext(), "Failed to send command for", Toast.LENGTH_SHORT).show();
		}
	}

	private void updateState(String[] lights, boolean state){
		if (!_lightsBound) return;
		for (String light : lights)
			_lightsService.updateState(light, state);
		send();
	}

	private void on(String... lights){
		updateState(lights, true);
	}

	private void off(String... lights){
		updateState(lights, false);
	}

	private void setBrightness(String light, int value, boolean send){
		if (!_lightsBound) return;
		_lightsService.updateBrightness(light, ((float)value)/100);
		if (send)
			send();
	}

	private void setBrightness(String light, int value){
		setBrightness(light, value, true);
	}

	private void setTimer(String light, NumberPicker minutesPicker, NumberPicker secondsPicker, boolean send) {
		if (!_lightsBound)
			return;

		int minutes = minutesPicker.getValue();
		int seconds = secondsPicker.getValue();

		int totalSeconds=(minutes*60)+seconds;

		_lightsService.updateTimer(light, totalSeconds*1000);

		if (send)
			send();
	}

	private void setTimer(String light, NumberPicker minutesPicker, NumberPicker secondsPicker) {
		setTimer(light, minutesPicker, secondsPicker, true);
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		hasPermissions();

		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

        findViewById(R.id.seating_on_button).setOnClickListener(view -> on("seating"));
        findViewById(R.id.seating_off_button).setOnClickListener(view -> off("seating"));

        findViewById(R.id.kitchen_on_button).setOnClickListener(view -> on("kitchen"));
        findViewById(R.id.kitchen_off_button).setOnClickListener(view -> off("kitchen"));

		findViewById(R.id.bathroom_on_button).setOnClickListener(view -> on("bathroom"));
		findViewById(R.id.bathroom_off_button).setOnClickListener(view -> off("bathroom"));

        findViewById(R.id.all_on_button).setOnClickListener(view -> on("seating", "kitchen", "bathroom"));
        findViewById(R.id.all_off_button).setOnClickListener(view -> off("seating", "kitchen", "bathroom"));

		findViewById(R.id.awning_on_button).setOnClickListener(view -> on("awning"));
		findViewById(R.id.awning_off_button).setOnClickListener(view -> off("awning"));

		((SeekBar)findViewById(R.id.seating_brightness)).setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
			@Override
			public void onProgressChanged(SeekBar seekBar, int i, boolean b) {

			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {

			}

			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				setBrightness("seating", seekBar.getProgress());
			}
		});
		((SeekBar)findViewById(R.id.kitchen_brightness)).setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
			@Override
			public void onProgressChanged(SeekBar seekBar, int i, boolean b) {

			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {

			}

			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				setBrightness("kitchen", seekBar.getProgress());
			}
		});
		((SeekBar)findViewById(R.id.bathroom_brightness)).setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
			@Override
			public void onProgressChanged(SeekBar seekBar, int i, boolean b) {

			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {

			}

			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				setBrightness("bathroom", seekBar.getProgress());
			}
		});
		((SeekBar)findViewById(R.id.awning_brightness)).setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
			@Override
			public void onProgressChanged(SeekBar seekBar, int i, boolean b) {

			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {

			}

			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				setBrightness("awning", seekBar.getProgress());
			}
		});
		((SeekBar)findViewById(R.id.all_brightness)).setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
			@Override
			public void onProgressChanged(SeekBar seekBar, int i, boolean b) {

			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {

			}

			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				int progress = seekBar.getProgress();
				setBrightness("seating", progress, false);
				setBrightness("kitchen", progress, false);
				setBrightness("bathroom", progress);
			}
		});

		NumberPicker seatingMinutes = findViewById(R.id.seating_timer_minutes);
		seatingMinutes.setMaxValue(30);
		seatingMinutes.setMinValue(0);
		NumberPicker seatingSeconds = findViewById(R.id.seating_timer_seconds);
		seatingSeconds.setMaxValue(59);
		seatingSeconds.setMinValue(0);
		findViewById(R.id.seating_set_timer).setOnClickListener(view -> setTimer("seating", seatingMinutes, seatingSeconds));

		NumberPicker kitchenMinutes = findViewById(R.id.kitchen_timer_minutes);
		kitchenMinutes.setMaxValue(30);
		kitchenMinutes.setMinValue(0);
		NumberPicker kitchenSeconds = findViewById(R.id.kitchen_timer_seconds);
		kitchenSeconds.setMaxValue(59);
		kitchenSeconds.setMinValue(0);
		findViewById(R.id.kitchen_set_timer).setOnClickListener(view -> setTimer("kitchen", kitchenMinutes, kitchenSeconds));

		NumberPicker bathroomMinutes = findViewById(R.id.bathroom_timer_minutes);
		bathroomMinutes.setMaxValue(30);
		bathroomMinutes.setMinValue(0);
		NumberPicker bathroomSeconds = findViewById(R.id.bathroom_timer_seconds);
		bathroomSeconds.setMaxValue(59);
		bathroomSeconds.setMinValue(0);
		findViewById(R.id.bathroom_set_timer).setOnClickListener(view -> setTimer("bathroom", bathroomMinutes, bathroomSeconds));

		NumberPicker allMinutes = findViewById(R.id.all_timer_minutes);
		allMinutes.setMaxValue(30);
		allMinutes.setMinValue(0);
		NumberPicker allSeconds = findViewById(R.id.all_timer_seconds);
		allSeconds.setMaxValue(59);
		allSeconds.setMinValue(0);
		findViewById(R.id.all_set_timer).setOnClickListener(view -> {
			setTimer("seating", allMinutes, allSeconds, false);
			setTimer("kitchen", allMinutes, allSeconds, false);
			setTimer("bathroom", allMinutes, allSeconds);
		});

		NumberPicker awningMinutes = findViewById(R.id.awning_timer_minutes);
		awningMinutes.setMaxValue(30);
		awningMinutes.setMinValue(0);
		NumberPicker awningSeconds = findViewById(R.id.awning_timer_seconds);
		awningSeconds.setMaxValue(59);
		awningSeconds.setMinValue(0);
		findViewById(R.id.awning_set_timer).setOnClickListener(view -> setTimer("awning", awningMinutes, awningSeconds));
	}

	@Override
	protected void onStart() {
		super.onStart();

		Intent intent = new Intent(this, LightsService.class);
		bindService(intent, _serviceConnection, Context.BIND_AUTO_CREATE);
		connect();
	}

	@Override
	protected void onStop() {
		super.onStop();
		disconnect();
		unbindService(_serviceConnection);
		_lightsBound = false;
	}


	@Override
	protected void onResume() {
		super.onResume();
		connect();
	}

	@Override
	protected void onPause() {
		super.onPause();
		disconnect();
	}

}